// Author: Pete Kirkham
// tiny quad store benchmark.
//
// read binary benchmark

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <iostream>

#define BLOCK_SIZE (16*1024)
#define MiB (1024*1024)

#include "utils/string_utils.h"

int main (int narg, char** argvp)
{
    if (narg < 2)
    {
        std::cout << "The name of the input file is required." << std::endl;
        return 1;
    }

    ssize_t max_read(std::numeric_limits<ssize_t>::max());

    if (narg > 2)
    {
        max_read = atoi(argvp[2]) * MiB;
    }

    // open the file
    int file(open(argvp[1], O_RDONLY));

    if (!file)
    {
        std::cout << "failed to open " << argvp[1] << std::endl;
        return 2;
    }

    // read in blocks
    int         buf[BLOCK_SIZE / 4];
    ssize_t     read_now;
    ssize_t     read_total(0);
    ssize_t     MiBMask((ssize_t(MiB)*8)-1);

    while ((read_now = read(file, buf, BLOCK_SIZE)) > 0)
    {
        read_total += read_now;

        if ((read_total & MiBMask) == 0)
        {
            if (read_total >= max_read) break;

            std::cout << "read " << read_total/MiB << " MiB." << std::endl;
        }
    }

    std::cout << "read " << read_total/MiB << " MiB." << std::endl;

    close(file);

    return 0;
}
